#include "ErrorBar.h"

#include <qwt_plot.h>
#include <qwt_curve.h>
#include <qwt_painter.h>
#include <qwt_double_rect.h>
#include <qpainter.h>
#include <qmessagebox.h>

QwtErrorPlotCurve::QwtErrorPlotCurve(QwtPlot *parent, const char *name):
    QwtPlotCurve(parent,name)
{
pen=QPen(Qt::black,2,Qt::SolidLine);
brush=QBrush(Qt::black);
cap=10;
type=1;
size=QSize(1,1);
plus=TRUE;
minus=TRUE;
through=FALSE;
}


void QwtErrorPlotCurve::draw(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{
    if ( !painter || dataSize() <= 0 )
        return;

    if (to < 0)
        to = dataSize() - 1;

    if ( verifyRange(from, to) > 0 )
    {
        painter->save();
        painter->setPen(pen);
        painter->setBrush(brush);
		drawErrorBars(painter, xMap, yMap, from, to);
        painter->restore();
    }
}

void QwtErrorPlotCurve::drawErrorBars(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{   
	int sh=size.height();
	int sw=size.width();

 	for (int i = from; i <= to; i++)
		{
		const int xi = xMap.transform(x(i));
		const int yi = yMap.transform(y(i));

		if (type==1)
			{
			const int yh = yMap.transform(y(i)+err[i]);
			const int yl = yMap.transform(y(i)-err[i]);
			const int yhl=(yh+yl-sh)/2-pen.width();
			const int ylh=(yh+yl+sh)/2+pen.width();

			if ((yl-yh)>sh)
				{
				if (plus)
					{
					QwtPainter::drawLine(painter,xi,yhl,xi,yh);
					QwtPainter::drawLine(painter,xi-cap/2,yh,xi+cap/2,yh);
					}
				if (minus)
					{
					QwtPainter::drawLine(painter,xi,ylh,xi,yl);
					QwtPainter::drawLine(painter,xi-cap/2,yl,xi+cap/2,yl);
					}
				if (through)
					QwtPainter::drawLine(painter,xi,yhl,xi,ylh);
				}
			}
		else if (type==0)
			{
			const int xp = xMap.transform(x(i)+err[i]);
			const int xm = xMap.transform(x(i)-err[i]);
			const int xpm=(xp+xm+sw)/2+pen.width();
			const int xmp=(xp+xm-sw)/2-pen.width();

			if ((xp-xm)>sw)
			{
			if (plus)
				{
				QwtPainter::drawLine(painter,xp,yi,xpm,yi);
				QwtPainter::drawLine(painter,xp,yi-cap/2,xp,yi+cap/2);
				}
			if (minus)
				{
				QwtPainter::drawLine(painter,xm,yi,xmp,yi);
				QwtPainter::drawLine(painter,xm,yi-cap/2,xm,yi+cap/2);
				}
			if (through)
				QwtPainter::drawLine(painter,xmp,yi,xpm,yi);
			}
			}

		}
}

bool QwtErrorPlotCurve::throughSymbol()
{
return through;
}

void QwtErrorPlotCurve::drawThroughSymbol(bool yes)
{
through=yes;
}

bool QwtErrorPlotCurve::minusSide()
{
return minus;
}

void QwtErrorPlotCurve::drawMinusSide(bool yes)
{
minus=yes;
}

bool QwtErrorPlotCurve::plusSide()
{
return plus;
}

void QwtErrorPlotCurve::drawPlusSide(bool yes)
{
plus=yes;
}

QwtArray<double> QwtErrorPlotCurve::errors()
{
return err;
}

void QwtErrorPlotCurve::setErrors(const QwtArray<double>&data)
{
err=data;
}

void QwtErrorPlotCurve::setSymbolSize(const QSize& sz)
{
size=sz;
}

int QwtErrorPlotCurve::direction()
{
return type;
}

void QwtErrorPlotCurve::setXErrors(bool yes)
{
if (yes) type=0;
else type=1;
}

int QwtErrorPlotCurve::capLength()
{
return cap;
}

void QwtErrorPlotCurve::setCapLength(int t)
{
cap=t;
}

int QwtErrorPlotCurve::width()
{
return pen.width ();
}

void QwtErrorPlotCurve::setWidth(int w)
{
pen.setWidth (w);
}

QColor QwtErrorPlotCurve::color()
{
return pen.color();
}

void QwtErrorPlotCurve::setColor(const QColor& c)
{
pen.setColor(c);
}

QwtDoubleRect QwtErrorPlotCurve::boundingRect() const
{
       QwtDoubleRect rect = QwtCurve::boundingRect();
	
		double max=err[0];
		for (int i=0;i<(int)err.size();i++)
			{
			if (err[i]>=max)	
				max=err[i];
			}
			
        if (type == 1)
			{
          	rect.setY1(rect.y1() - max);
          	rect.setY2(rect.y2() + max);
			}
        else
			{
          	rect.setX1(rect.x1() - max);
          	rect.setX2(rect.x2() + max);
			}
			
return rect;
}
