#include "ImageMarker.h"

#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <qwt_plot.h>
#include <qwt_scale.h>
#include <qwt_painter.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_layout.h>
#include <qwt_layout_metrics.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

ImageMarker::ImageMarker(const QPixmap& p, QwtPlot *plot):
    QwtPlotMarker(plot)
{
pic=p;
}

void ImageMarker::draw(QPainter *p, int, int, const QRect&)
{
	QwtPlot *plot = (QwtPlot *)parentPlot();
	QPaintDeviceMetrics pdmFrom(plot);
	QPaintDeviceMetrics pdmTo(p->device());
	
	double scaleFactorX=pdmTo.logicalDpiX()*1.0/pdmFrom.logicalDpiX();
	double scaleFactorY=pdmTo.logicalDpiY()*1.0/pdmFrom.logicalDpiY();
				
	if (scaleFactorX !=1 || scaleFactorY !=1)
		{
		const QwtDiMap xMap = plot->canvasMap(QwtPlot::xBottom);
		const QwtDiMap yMap = plot->canvasMap(QwtPlot::yLeft);
	
		double r0_x=xMap.invTransform(origin.x());
		double r0_y=yMap.invTransform(origin.y());	
		double r1_x=xMap.invTransform(origin.x()+picSize.width());
		double r1_y=yMap.invTransform(origin.y()+picSize.height());
			
		QwtDiMap map=scaleMap(p,QwtPlot::xBottom);
		int x0=map.transform(r0_x);
		int x1=map.transform(r1_x);
	
	    map=scaleMap(p,QwtPlot::yLeft);		
		int y0=map.transform(r0_y);
		int y1=map.transform(r1_y);

		p->drawPixmap(QRect(QPoint(x0,y0),QPoint(x1,y1)),pic);			
		}		
	else	
		p->drawPixmap(QRect(origin,picSize),pic);
}

void ImageMarker::setSize(const QSize& size)
{
if (picSize == size)
	return;
picSize=size;
}

void ImageMarker::setOrigin(const QPoint& p)
{
if (origin == p)
	return;
origin=p;
}

QPoint ImageMarker::getOrigin()
{
return origin;	
}

QwtDiMap ImageMarker::scaleMap(QPainter *p,int axis) 
{
QwtPlot *plot = (QwtPlot *)parentPlot();
QwtPlotLayout *pl=plot->plotLayout ();
 
QPaintDeviceMetrics pdmFrom(plot);
QPaintDeviceMetrics pdmTo(p->device());	
	
QwtMetricsMap metricsMap;
metricsMap.setMetrics(pdmFrom,pdmTo);
			
QwtDiMap map;
double from,to;	
if (plot->axisEnabled(axis))
	{
	const QwtScaleDiv *scaleDiv =plot->axisScale(axis);
	map.setDblRange(scaleDiv->lBound(),
				scaleDiv->hBound(), scaleDiv->logScale());

	int sDist = plot->axis(axis)->startBorderDist();
	int eDist = plot->axis(axis)->endBorderDist();
	
	const QRect &scaleRect = pl->scaleRect(axis);
		
	if (axis==QwtPlot::xBottom)
		{
		from= metricsMap.layoutToDeviceX(scaleRect.left() + sDist);
		to = metricsMap.layoutToDeviceX(scaleRect.right() - eDist);
		}
	else if (axis==QwtPlot::yLeft)
		{
		from = metricsMap.layoutToDeviceY(scaleRect.bottom() - sDist);
    	to = metricsMap.layoutToDeviceY(scaleRect.top() + eDist);
		}
	}
else
	{
	const int margin = pl->canvasMargin(axis);
		
    const QRect &canvasRect = pl->canvasRect();
    if ( axis ==QwtPlot::yLeft)
         {
		 QwtDiMap yMap = plot->canvasMap(QwtPlot::yLeft);
		 map.setDblRange(yMap.d1(),yMap.d2(),yMap.logarithmic());
			 
         from = metricsMap.layoutToDeviceX(canvasRect.bottom() - margin);
         to = metricsMap.layoutToDeviceX(canvasRect.top() + margin);
         }
    else
        {
		QwtDiMap xMap = plot->canvasMap(QwtPlot::xBottom);
		map.setDblRange(xMap.d1(),xMap.d2(),xMap.logarithmic());
			
        from = metricsMap.layoutToDeviceY(canvasRect.left() + margin);
        to = metricsMap.layoutToDeviceY(canvasRect.right() - margin);
        }
	}      	
map.setIntRange(qwtInt(from), qwtInt(to));	
return map;	
}
